@@ -1,26 +1,6 @@
 package OpenTracing::Role::Span;
 
-=head1 NAME
-
-OpenTracing::Role::Span - Role for OpenTracing implementations.
-
-=head1 SYNOPSIS
-
-    package OpenTracing::Implementation::MyBackendService::Span;
-    
-    use Moo;
-    
-    ...
-    
-    with 'OpenTracing::Role::Span'
-    
-    1;
-
-=cut
-
-
-
-our $VERSION = 'v0.70.1';
+our $VERSION = 'v0.80.0';
 
 
 
@@ -28,52 +8,32 @@ use Moo::Role;
 use MooX::HandlesVia;
 
 use Carp;
-use Time::HiRes qw/gettimeofday/;
-use Types::Standard qw/HashRef Num Object Str Value/;
-use Types::Interface qw/ObjectDoesInterface/;
-
-
-
-=head1 DESCRIPTION
-
-This is a Role for OpenTracing implenetations that are compliant with the
-L<OpenTracing::Interface>.
-
-With the exception of calls to C<get_context()> (which are always allowed),
-C<finish()> must be the last call made to any span instance, and to do otherwise
-leads to undefined behavior (but not returning an exception).
-
-=cut
-
-
+use OpenTracing::Types qw/:types :is/;
+use Time::HiRes qw/time/;
+use Types::Standard qw/CodeRef HashRef Maybe Num Object Str Value/;
+use Types::Common::Numeric qw/PositiveOrZeroNum/;
 
 has operation_name => (
     is              => 'rwp',
     isa             => Str,
     required        => 1,
 #   writer          => 'overwrite_operation_name',
-#   reader          => 'get_operation_name', # not sure, it's not in the Interface
+    reader          => 'get_operation_name', # it's not in the Interface
 );
 
-
-
 has start_time => (
     is              => 'ro',
-    isa             => Num,
+    isa             => PositiveOrZeroNum,
     default         => sub { epoch_floatingpoint() }
 );
 
-
-
 has finish_time => (
     is              => 'rwp',
-    isa             => Num,
+    isa             => PositiveOrZeroNum,
     predicate       => 'has_finished',
     init_arg        => undef,
 );
 
-
-
 has tags => (
     is              => 'rwp',
     isa             => HashRef[Value],
@@ -84,18 +44,14 @@ has tags => (
     default         => sub{ {} },
 );
 
-
-
 has context => (
     is              => 'ro',
-    isa             => ObjectDoesInterface['OpenTracing::Interface::SpanContext'],
+    isa             => SpanContext,
     reader          => 'get_context',
 #   writer          => '_set_context',
     required        => 1, # either from Span->get_context or SpanContext self
 );
 
-
-
 sub overwrite_operation_name {
     my $self = shift;
     
@@ -109,24 +65,23 @@ sub overwrite_operation_name {
     return $self
 }
 
-
-
 sub finish {
     my $self = shift;
     
-    croak "Tag has already been finished"
+    croak "Span has already been finished"
         if $self->has_finished;
     
     my $epoch_timestamp = shift // epoch_floatingpoint();
     
     $self->_set_finish_time( $epoch_timestamp );
     
+    $self->on_finish->( $self )
+        if $self->has_on_finish;
+    
     return $self
 }
 
-
-
-sub set_tag {
+sub add_tag {
     my $self = shift;
     
     croak "Can't set a tag on an already finished span"
@@ -135,12 +90,24 @@ sub set_tag {
     my $key = shift;
     my $value = shift;
     
-    $self->{ tags }->{ $key } = $value;
+    $self->add_tags( $key => $value );
     
     return $self
 }
 
-
+sub add_tags {
+    my $self = shift;
+    
+    croak "Can't set a tag on an already finished span"
+        if $self->has_finished;
+    
+    my %tags = @_;
+    $self->_set_tags(
+        { $self->get_tags, %tags }
+    );
+    
+    return $self
+}
 
 sub log_data {
     my $self = shift;
@@ -155,9 +122,7 @@ sub log_data {
     return $self
 }
 
-
-
-sub set_baggage_item {
+sub add_baggage_item {
     my $self = shift;
     
     croak "Can't set baggage-items on an already finished span"
@@ -166,15 +131,12 @@ sub set_baggage_item {
     my $key = shift;
     my $value = shift;
     
-    my $new_context = $self->get_context()->with_baggage_item( $key, $value );
-    $self->_set_context( $new_context );
+    $self->add_baggage_items( $key => $value );
     
     return $self
 }
 
-
-
-sub set_baggage_items {
+sub add_baggage_items {
     my $self = shift;
     
     croak "Can't set baggage-items on an already finished span"
@@ -188,8 +150,6 @@ sub set_baggage_items {
     return $self
 }
 
-
-
 sub get_baggage_item {
     my $self = shift;
     my $key = shift;
@@ -197,7 +157,11 @@ sub get_baggage_item {
     return $self->get_context()->get_baggage_item( $key )
 }
 
-
+sub get_baggage_items {
+    my $self = shift;
+    
+    return $self->get_context()->get_baggage_items
+}
 
 sub duration { 
     my $self = shift;
@@ -206,20 +170,39 @@ sub duration {
         or croak
             "Span has not been started: ['"
             .
-            $self->operation_name
+            ( $self->get_operation_name || "'undef'" )
             .
             "'] ... how did you do that ?";
     my $finish_time = $self->{ finish_time }
         or croak
             "Span has not been finished: ['"
             .
-            $self->operation_name
+            ( $self->get_operation_name || "'undef'" )
             .
             "'] ... yet!";
     
     return $finish_time - $start_time
 }
 
+has child_of => (
+    is => 'ro',
+    isa => Span | SpanContext,
+    required => 1,
+);
+
+sub parent_span_id {
+    my $self = shift;
+    
+    my $parent = $self->{ child_of };
+    return unless is_Span( $parent );
+    
+    return $parent->span_id
+}
+#
+# This may not be the right way to implement it, for the `child_of` attribute
+# may not be such a good idea, maybe it should use references, but not sure how
+# those are used
+
 
 
 # _set_context
@@ -239,16 +222,28 @@ sub _set_context {
     return $self
 }
 
+has on_finish => (
+    is              => 'ro',
+    isa             => Maybe[CodeRef],
+    predicate       => 1,
+);
 
-
-=head2 epoch_floatingpoint
-
-Well, returns the time since 'epoch' with fractional seconds, as floating-point.
-
-=cut
+sub DEMOLISH {
+    my $self = shift;
+    my $in_global_destruction = shift;
+    
+    return if $self->has_finished;
+    
+#   carp "Span not programmatically finished before being demolished";
+    
+    $self->finish( )
+        unless $in_global_destruction;
+    
+    return
+}
 
 sub epoch_floatingpoint {
-    return scalar gettimeofday()
+    return time()
 }
 #
 # well, this is a bit off a silly idea:
@@ -268,5 +263,3 @@ BEGIN {
 
 
 1;
-
-
